/**
 *
 * Project: Crypto, File: RandomString.java
 *
 * Copyright 2019 Locusworks LLC.
 * All rights reserved. Federal copyright law prohibits unauthorized reproduction by
 * any means and imposes fines up to $25,000 for violation.  No part of this material
 * may be reproduced, transmitted, transcribed, stored in a retrieval system, copied,
 * modified, duplicated, adapted or translated into another program language in any
 * form or by any means, electronic, mechanical, photocopying, recording, or
 * otherwise, without the prior written permission from Locusworks. Locusworks
 * affirms that Crypto(R) software and data is subject to United States
 * Government Purpose Rights.  Contact Locusworks, 1313 Lawnview Drive
 * Forney TX 75126, (802) 488-0438, for commercial licensing opportunities.
 *
 * IN NO EVENT SHALL LOCUSWORKS BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LOCUSWORKS HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  NO RESPONSIBILITY IS ASSUMED BY
 * LOCUSWORKS FOR ITS USE, OR FOR ANY INFRINGEMENTS OF PATENTS OR OTHER RIGHTS OF
 * THIRD PARTIES RESULTING FROM ITS USE.  LOCUSWORKS SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE AND
 * ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED HEREUNDER IS PROVIDED "AS
 * IS". LOCUSWORKS HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 */
package net.locusworks.crypto.utils;

import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;
import java.util.Objects;
import java.util.Random;

public class RandomString {

  public static final String LOWER = "abcdefghijklmnopqrstuvwxyz";
  public static final String UPPER = LOWER.toUpperCase();
  public static final String DIGITS = "0123456789";
  
  public static final String ALPHA_NUMERIC = LOWER + UPPER + DIGITS;
  
  private Random random;
  
  private char[] symbols;
  private int length;
  
  private static RandomString instance;
  
  private RandomString(Integer length) {
    this(length, new SecureRandom());
  }
  
  private RandomString(Integer length, Random random) {
    this(length, random, ALPHA_NUMERIC);
  }
  
  private RandomString(Integer length, Random random, String symbols) {
    if (length < 1) throw new IllegalArgumentException("Length has to be greater than 1");
    if (symbols.length() < 2) throw new IllegalArgumentException("Symbols need to be greater than 2");
    this.random = Objects.requireNonNull(random);
    this.symbols = symbols.toCharArray();
  }
  
  private synchronized final void setRandom(Random random) {
    this.random = random;
  }

  private synchronized final void setLength(int length) {
    this.length = length;
  }

  public String nextString() {
    char[] buffer = new char[length];
    for (int index = 0; index < buffer.length; index++) {
     buffer[index] = symbols[random.nextInt(symbols.length)]; 
    }
    return new String(buffer);
  }
  
  public static String getString(Integer length) {
    if (instance == null) {
      instance = new RandomString(length);
    }
    instance.setLength(length);
    return instance.nextString();
  }
  
  public static String getString(Integer length, Random random) {
    if (instance == null) {
      instance = new RandomString(length);
    }
    instance.setLength(length);
    instance.setRandom(random);
    return instance.nextString();
  }
  
  public static byte[] getBytes(Integer length) {
    return getString(length).getBytes(StandardCharsets.UTF_8);
  }
  
  public static byte[] getBytes(Integer length, Random random) {
    return getString(length, random).getBytes(StandardCharsets.UTF_8);
  }
}
